{#
    basic/page.html
    ~~~~~~~~~~~~~~~

    Master template for simple pages.

    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.
#}
{%- extends "layout.html" %}
{% block body %}
  {{ body }}
{% endblock %}
