{#
    basic/searchbox.html
    ~~~~~~~~~~~~~~~~~~~~

    Sphinx sidebar template: quick search box.

    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.
#}
{%- if pagename != "search" and builder != "singlehtml" %}
<div id="searchbox" style="display: none" role="search">
  <h3 id="searchlabel">{{ _('Quick search') }}</h3>
    <div class="searchformwrapper">
    <form class="search" action="{{ pathto('search') }}" method="get">
      <input type="text" name="q" aria-labelledby="searchlabel" />
      <input type="submit" value="{{ _('Go') }}" />
    </form>
    </div>
</div>
<script>$('#searchbox').show(0);</script>
{%- endif %}
