Documentation.addTranslations({
    "locale": "ko",
    "messages": {
        "%(filename)s &#8212; %(docstitle)s": "",
        "&#169; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "",
        "&#169; Copyright %(copyright)s.": "",
        ", in ": ", \ubb38\uc11c - ",
        "About these documents": "\uc774 \ubb38\uc11c \uc815\ubcf4",
        "Automatically generated list of changes in version %(version)s": "\ubc84\uc804 %(version)s\uc758 \ubcc0\uacbd \uc0ac\ud56d (\uc774 \ubaa9\ub85d\uc740 \uc790\ub3d9\uc73c\ub85c \uc0dd\uc131\ud569\ub2c8\ub2e4)",
        "C API changes": "C API\uc5d0 \ub300\ud55c \ubcc0\uacbd",
        "Changes in Version %(version)s &#8212; %(docstitle)s": "",
        "Collapse sidebar": "\uc0ac\uc774\ub4dc\ubc14 \ub2eb\uae30",
        "Complete Table of Contents": "\uc885\ud569 \ubaa9\ucc28",
        "Contents": "\ub0b4\uc6a9",
        "Copyright": "\uc800\uc791\uad8c",
        "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "<a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s \ubc84\uc804\uc73c\ub85c \uc0dd\uc131\ub418\uc5c8\uc2b5\ub2c8\ub2e4.",
        "Expand sidebar": "\uc0ac\uc774\ub4dc\ubc14 \uc5f4\uae30",
        "Full index on one page": "\ud55c \ud398\uc774\uc9c0\uc5d0 \uc804\uccb4 \uc0c9\uc778 \ubcf4\uae30",
        "General Index": "\uc804\uccb4 \uc0c9\uc778",
        "Global Module Index": "\ubaa8\ub4c8 \ucd1d \uc0c9\uc778",
        "Go": "\uc774\ub3d9",
        "Hide Search Matches": "\uac80\uc0c9 \uacb0\uacfc \uc228\uae30\uae30",
        "Index": "\uc0c9\uc778",
        "Index &ndash; %(key)s": "",
        "Index pages by letter": "\uc54c\ud30c\ubcb3\ubcc4 \uc0c9\uc778",
        "Indices and tables:": "\uc0c9\uc778 \ubc0f \ud45c \ubaa9\ub85d:",
        "Last updated on %(last_updated)s.": "\ucd5c\uc885 \uc5c5\ub370\uc774\ud2b8: %(last_updated)s",
        "Library changes": "\ub77c\uc774\ube0c\ub7ec\ub9ac\uc5d0 \ub300\ud55c \ubcc0\uacbd",
        "Navigation": "\ud0d0\uc0c9",
        "Next topic": "\ub2e4\uc74c \ud56d\ubaa9",
        "Other changes": "\ub2e4\ub978 \ubcc0\uacbd \uc0ac\ud56d",
        "Overview": "\uac1c\uc694",
        "Permalink to this definition": "\uc815\uc758 \uc8fc\uc18c",
        "Permalink to this headline": "\uc81c\ubaa9 \uc8fc\uc18c",
        "Please activate JavaScript to enable the search\n    functionality.": "\uac80\uc0c9 \uae30\ub2a5\uc744 \uc0ac\uc6a9\ud558\ub824\uba74 JavaScript\ub97c \ud65c\uc131\ud654\ud558\uc2ed\uc2dc\uc624.",
        "Preparing search...": "\uac80\uc0c9 \uc900\ube44 \uc911\u2026",
        "Previous topic": "\uc774\uc804 \ud56d\ubaa9",
        "Quick search": "\ube60\ub978 \uac80\uc0c9",
        "Search": "\uac80\uc0c9",
        "Search Page": "\uac80\uc0c9 \ud398\uc774\uc9c0",
        "Search Results": "\uac80\uc0c9 \uacb0\uacfc",
        "Search finished, found %s page(s) matching the search query.": "",
        "Search within %(docstitle)s": "%(docstitle)s\uc5d0\uc11c \ucc3e\uae30",
        "Searching": "\uac80\uc0c9 \uc911",
        "Searching for multiple words only shows matches that contain\n    all words.": "",
        "Show Source": "\uc18c\uc2a4 \ubcf4\uae30",
        "Table of Contents": "\ubaa9\ucc28",
        "This Page": "\ud604\uc7ac \ubb38\uc11c",
        "Welcome! This is": "",
        "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "",
        "all functions, classes, terms": "\ud568\uc218, \ud074\ub798\uc2a4 \ubc0f \uc6a9\uc5b4 \uac1c\uad00",
        "can be huge": "\ud070 \uacbd\uc6b0\uac00 \uc788\uc73c\ubbc0\ub85c \uc8fc\uc758",
        "last updated": "\ucd5c\uc885 \uc5c5\ub370\uc774\ud2b8",
        "lists all sections and subsections": "\uc601\uc5ed\ubcc4 \ubaa9\ucc28",
        "next chapter": "\ub2e4\uc74c \uc7a5",
        "previous chapter": "\uc774\uc804 \uc7a5",
        "quick access to all modules": "\ubaa8\ub4e0 \ubaa8\ub4c8 \uc870\uacac\ud45c",
        "search": "\uac80\uc0c9",
        "search this documentation": "\ubb38\uc11c \uac80\uc0c9",
        "the documentation for": ""
    },
    "plural_expr": "0"
});