Documentation.addTranslations({
    "locale": "pt_BR",
    "messages": {
        "%(filename)s &#8212; %(docstitle)s": "%(filename)s &#8212; %(docstitle)s",
        "&#169; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&#169; <a href=\"%(path)s\">Copyright</a> %(copyright)s.",
        "&#169; Copyright %(copyright)s.": "&#169; Copyright %(copyright)s.",
        ", in ": ", em ",
        "About these documents": "Sobre esses documentos",
        "Automatically generated list of changes in version %(version)s": "Lista de altera\u00e7\u00f5es na vers\u00e3o %(version)s, gerada automaticamente",
        "C API changes": "Altera\u00e7\u00f5es na API C",
        "Changes in Version %(version)s &#8212; %(docstitle)s": "Modifica\u00e7\u00f5es na vers\u00e3o %(version)s &#8212; %(docstitle)s",
        "Collapse sidebar": "Recolher painel lateral",
        "Complete Table of Contents": "Tabela Completa dos Conte\u00fados",
        "Contents": "Conte\u00fados",
        "Copyright": "Copyright",
        "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Criado usando <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.",
        "Expand sidebar": "Expandir painel lateral",
        "Full index on one page": "\u00cdndice completo em p\u00e1gina \u00fanica",
        "General Index": "\u00cdndice Geral",
        "Global Module Index": "\u00cdndice Global de M\u00f3dulos",
        "Go": "Ir",
        "Hide Search Matches": "Esconder Resultados da Busca",
        "Index": "\u00cdndice",
        "Index &ndash; %(key)s": "\u00cdndice &ndash; %(key)s",
        "Index pages by letter": "P\u00e1ginas de \u00edndice por letra",
        "Indices and tables:": "\u00cdndices e Tabelas:",
        "Last updated on %(last_updated)s.": "\u00daltima atualiza\u00e7\u00e3o em %(last_updated)s.",
        "Library changes": "Altera\u00e7\u00f5es na biblioteca",
        "Navigation": "Navega\u00e7\u00e3o",
        "Next topic": "Pr\u00f3ximo t\u00f3pico",
        "Other changes": "Outras altera\u00e7\u00f5es",
        "Overview": "Vis\u00e3o geral",
        "Permalink to this definition": "Link permanente para esta defini\u00e7\u00e3o",
        "Permalink to this headline": "Link permanente para este t\u00edtulo",
        "Please activate JavaScript to enable the search\n    functionality.": "Por favor, ativar JavaScript para habilitar a\nfuncionalidade de busca.",
        "Preparing search...": "Preparando a busca...",
        "Previous topic": "T\u00f3pico anterior",
        "Quick search": "Busca r\u00e1pida",
        "Search": "Buscar",
        "Search Page": "P\u00e1gina de Busca",
        "Search Results": "Resultados da Busca",
        "Search finished, found %s page(s) matching the search query.": "Busca conclu\u00edda. %s p\u00e1gina(s) que atendem a consulta.",
        "Search within %(docstitle)s": "Pesquisar dentro de %(docstitle)s",
        "Searching": "Buscando",
        "Searching for multiple words only shows matches that contain\n    all words.": "Pesquisando por v\u00e1rias palavras s\u00f3 mostra correspond\u00eancias\nque cont\u00eam todas as palavras.",
        "Show Source": "Exibir Fonte",
        "Table of Contents": "Tabela de Conte\u00fado",
        "This Page": "Essa P\u00e1gina",
        "Welcome! This is": "Bem Vindo! \u00c9 isso a\u00ed.",
        "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "Sua busca n\u00e3o encontrou nenhum documento. Por favor, confirme se todas as palavras est\u00e3o grafadas corretamente e se voc\u00ea selecionou categorias suficientes.",
        "all functions, classes, terms": "todas fun\u00e7\u00f5es, classes, termos",
        "can be huge": "pode ser enorme",
        "last updated": "\u00faltima atualiza\u00e7\u00e3o",
        "lists all sections and subsections": "Listar todas se\u00e7\u00f5es e subse\u00e7\u00f5es",
        "next chapter": "pr\u00f3ximo cap\u00edtulo",
        "previous chapter": "cap\u00edtulo anterior",
        "quick access to all modules": "acesso r\u00e1pido para todos os m\u00f3dulos",
        "search": "buscar",
        "search this documentation": "Buscar nessa documenta\u00e7\u00e3o",
        "the documentation for": "documenta\u00e7\u00e3o para"
    },
    "plural_expr": "(n > 1)"
});