Documentation.addTranslations({
    "locale": "id",
    "messages": {
        "%(filename)s &#8212; %(docstitle)s": "%(filename)s &#8212; %(docstitle)s",
        "&#169; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&#169; <a href=\"%(path)s\">Hak cipta</a> %(copyright)s.",
        "&#169; Copyright %(copyright)s.": "&#169; Hak cipta %(copyright)s.",
        ", in ": ", di",
        "About these documents": "Tentang dokumen ini",
        "Automatically generated list of changes in version %(version)s": "Daftar perubahan dibuat otomatis untuk versi %(version)s",
        "C API changes": "Perubahan API C",
        "Changes in Version %(version)s &#8212; %(docstitle)s": "Perubahan pada Versi %(version)s &#8212; %(docstitle)s",
        "Collapse sidebar": "Tutup sidebar",
        "Complete Table of Contents": "Daftar Isi Lengkap",
        "Contents": "Konten",
        "Copyright": "Hak Cipta",
        "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Dibuat menggunakan <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.",
        "Expand sidebar": "Buka sidebar",
        "Full index on one page": "Index penuh dalam satu halaman",
        "General Index": "Indeks Umum",
        "Global Module Index": "Index Modul Global",
        "Go": "Go",
        "Hide Search Matches": "Sembunyikan Hasil Pencarian",
        "Index": "Indeks",
        "Index &ndash; %(key)s": "Index &ndash; %(key)s",
        "Index pages by letter": "Index halaman berdasarkan huruf",
        "Indices and tables:": "Indeks dan tabel:",
        "Last updated on %(last_updated)s.": "Terakhir diperbarui pada %(last_updated)s.",
        "Library changes": "Perubahan library",
        "Navigation": "Navigasi",
        "Next topic": "Topik berikutnya",
        "Other changes": "Perubahan lain",
        "Overview": "Tinjauan",
        "Permalink to this definition": "Link permanen untuk definisi ini",
        "Permalink to this headline": "Link permanen untuk headline ini",
        "Please activate JavaScript to enable the search\n    functionality.": "Tolong aktifkan JavaScript untuk melakukan pencarian.\n ",
        "Preparing search...": "Penyiapkan pencarian...",
        "Previous topic": "Topik sebelumnya",
        "Quick search": "Pencarian cepat",
        "Search": "Pencarian",
        "Search Page": "Pencarian Halaman",
        "Search Results": "Hasil Pencarian",
        "Search finished, found %s page(s) matching the search query.": "Pencarian selesai, menemukan %s halaman yang cocok dengan kueri pencarian.",
        "Search within %(docstitle)s": "Pencarian dalam %(docstitle)s",
        "Searching": "Pencarian",
        "Searching for multiple words only shows matches that contain\n    all words.": "Mencari beberapa kata hanya menunjukkan kecocokan yang mengandung\n \u00a0\u00a0 semua kata.",
        "Show Source": "Lihat Sumber",
        "Table of Contents": "Daftar Isi",
        "This Page": "Halaman Ini",
        "Welcome! This is": "Selamat Datang! Ini adalah",
        "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "Tidak ada dokumen yang cocok dengan pencarian anda. Pastikan semua kata ditulis dengan benar dan sudah memilih cukup kategori.",
        "all functions, classes, terms": "semua fungsi, class, term",
        "can be huge": "dapat menjadi besar",
        "last updated": "terakhir diperbarui",
        "lists all sections and subsections": "daftar semua seksi dan subseksi",
        "next chapter": "bab berikutnya",
        "previous chapter": "bab sebelum",
        "quick access to all modules": "akses cepat semua modul",
        "search": "pencarian",
        "search this documentation": "pencarian pada dokumentasi ini",
        "the documentation for": "dokumentasi untuk"
    },
    "plural_expr": "0"
});