Documentation.addTranslations({
    "locale": "da",
    "messages": {
        "%(filename)s &#8212; %(docstitle)s": "%(filename)s &#8212; %(docstitle)s",
        "&#169; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&#169; <a href=\"%(path)s\">Ophavsret</a> %(copyright)s.",
        "&#169; Copyright %(copyright)s.": "&#169; Ophavsret %(copyright)s.",
        ", in ": ", i",
        "About these documents": "Om disse dokumenter",
        "Automatically generated list of changes in version %(version)s": "Automatisk oprettet liste af \u00e6ndringer i version %(version)s",
        "C API changes": "\u00c6ndringer i C-API",
        "Changes in Version %(version)s &#8212; %(docstitle)s": "\u00c6ndringer i version %(version)s &#8212; %(docstitle)s",
        "Collapse sidebar": "Sammenfold sidebj\u00e6lke",
        "Complete Table of Contents": "Fuldst\u00e6ndig indholdsfortegnelse",
        "Contents": "Indhold",
        "Copyright": "Ophavsret",
        "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Bygget med <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.",
        "Expand sidebar": "Udfold sidebj\u00e6lke",
        "Full index on one page": "Fuldt indeks p\u00e5 \u00e9n side",
        "General Index": "Generelt indeks",
        "Global Module Index": "Globalt modulindeks",
        "Go": "S\u00f8g",
        "Hide Search Matches": "Skjul s\u00f8geresultater",
        "Index": "Indeks",
        "Index &ndash; %(key)s": "Indeks &ndash; %(key)s",
        "Index pages by letter": "Indeks\u00e9r sider efter bogstav",
        "Indices and tables:": "Indeks og tabeller:",
        "Last updated on %(last_updated)s.": "Sidst opdateret %(last_updated)s.",
        "Library changes": "Biblioteks\u00e6ndringer",
        "Navigation": "Navigation",
        "Next topic": "N\u00e6ste emne",
        "Other changes": "Andre \u00e6ndringer",
        "Overview": "Oversigt",
        "Permalink to this definition": "Permalink til denne definition",
        "Permalink to this headline": "Permalink til denne overskrift",
        "Please activate JavaScript to enable the search\n    functionality.": "Aktiv\u00e9r venligst JavaScript for at aktivere\n    s\u00f8gefunktionalitet.",
        "Preparing search...": "Forbereder s\u00f8gning...",
        "Previous topic": "Forrige emne",
        "Quick search": "Hurtig s\u00f8gning",
        "Search": "S\u00f8g",
        "Search Page": "S\u00f8geside",
        "Search Results": "S\u00f8geresultater",
        "Search finished, found %s page(s) matching the search query.": "S\u00f8gning f\u00e6rdig, fandt %s sider der matcher s\u00f8geforesp\u00f8rgslen.",
        "Search within %(docstitle)s": "S\u00f8g i %(docstitle)s",
        "Searching": "S\u00f8ger",
        "Searching for multiple words only shows matches that contain\n    all words.": "",
        "Show Source": "Vis kilde",
        "Table of Contents": "",
        "This Page": "Denne side",
        "Welcome! This is": "Velkommen! Dette er",
        "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "Din s\u00f8gning matchede ikke nogen dokumenter. Sikr dig at alle ord er stavet korrekt og at du har valgt nok kategorier.",
        "all functions, classes, terms": "alle funktioner, klasser, begreber",
        "can be huge": "kan v\u00e6re enormt",
        "last updated": "sidst opdateret",
        "lists all sections and subsections": "viser alle afsnit og underafsnit",
        "next chapter": "n\u00e6ste kapitel",
        "previous chapter": "forrige kapitel",
        "quick access to all modules": "hurtig adgang til alle moduler",
        "search": "s\u00f8g",
        "search this documentation": "s\u00f8g i denne dokumentation",
        "the documentation for": "dokumentationen for"
    },
    "plural_expr": "(n != 1)"
});