Documentation.addTranslations({
    "locale": "sl",
    "messages": {
        "%(filename)s &#8212; %(docstitle)s": "",
        "&#169; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "",
        "&#169; Copyright %(copyright)s.": "",
        ", in ": "",
        "About these documents": "O dokumentih",
        "Automatically generated list of changes in version %(version)s": "Avtomatsko generiran seznam sprememb v verziji %(version)s",
        "C API changes": "C API spremembe",
        "Changes in Version %(version)s &#8212; %(docstitle)s": "",
        "Collapse sidebar": "",
        "Complete Table of Contents": "Popoln Seznam Vsebine",
        "Contents": "",
        "Copyright": "Vse pravice pridr\u017eane",
        "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Narejeno s <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.",
        "Expand sidebar": "",
        "Full index on one page": "Poln indeks na eni strani",
        "General Index": "Splo\u0161ni abecedni seznam",
        "Global Module Index": "Splo\u0161en seznam modulov",
        "Go": "Potrdi",
        "Hide Search Matches": "Skrij resultate iskanja",
        "Index": "Abecedni seznam",
        "Index &ndash; %(key)s": "Seznam &ndash; %(key)s",
        "Index pages by letter": "Indeksiraj strani po \u010drki",
        "Indices and tables:": "Kazalo in seznami:",
        "Last updated on %(last_updated)s.": "Zadnji\u010d posodobljeno %(last_updated)s.",
        "Library changes": "Spremembe knji\u017enice",
        "Navigation": "Navigacija",
        "Next topic": "Naslednja tema",
        "Other changes": "Ostale spremembe",
        "Overview": "Pregled",
        "Permalink to this definition": "Povezava na to definicijo",
        "Permalink to this headline": "Povezava na naslov",
        "Please activate JavaScript to enable the search\n    functionality.": "Za pravilno delovanje Iskanja morete vklopiti\n    JavaScript.",
        "Preparing search...": "",
        "Previous topic": "Prej\u0161nja tema",
        "Quick search": "Hitro iskanje",
        "Search": "I\u0161\u010di",
        "Search Page": "Iskalnik",
        "Search Results": "Rezultati Iskanja",
        "Search finished, found %s page(s) matching the search query.": "",
        "Search within %(docstitle)s": "I\u0161\u010di med %(docstitle)s",
        "Searching": "",
        "Searching for multiple words only shows matches that contain\n    all words.": "",
        "Show Source": "Prika\u017ei izvorno kodo",
        "Table of Contents": "",
        "This Page": "Trenutna stran",
        "Welcome! This is": "",
        "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "",
        "all functions, classes, terms": "vse funkcije, razredi, izrazi",
        "can be huge": "lahko je veliko",
        "last updated": "",
        "lists all sections and subsections": "prikazi vse sekcije in podsekcije",
        "next chapter": "naslednje poglavje",
        "previous chapter": "prej\u0161nje poglavje",
        "quick access to all modules": "hiter dostop do vseh modulov",
        "search": "i\u0161\u010di",
        "search this documentation": "i\u0161\u010di po dokumentaciji",
        "the documentation for": ""
    },
    "plural_expr": "(n%100==1 ? 0 : n%100==2 ? 1 : n%100==3 || n%100==4 ? 2 : 3)"
});